#!/usr/bin/perl -w

my $enctype = "-e aes256-cts:normal";

my $keyfiledir = "/root/krb5-hostkeys";
-d $keyfiledir or mkdir $keyfiledir;
system("/usr/bin/find $keyfiledir -ctime +90 -print -delete");

############################################################
use POSIX;

@ARGV or ( print "usage: krb5_hostkey_generator.pl <ip-range>\n" and exit );

$domainname = `hostname -d`;
chomp $domainname;
$domainname or die "Failed to retrieve domainname";

$addresses = join " ", @ARGV;
print STDERR "Generating krb5 hostkeys for: $addresses\n";
open SCAN , "nmap -sP $addresses 2>/dev/null |" or die "Unable to start nmap: $!";
%hostlist = ();
while (<SCAN>){
  /Nmap scan report for ([\w\.\-]+) \((\d+.\d+.\d+.\d+)\)/
    and print "found $1, $2\n"
      and $hostlist{$1} = $2;
}
close SCAN;

foreach $host (sort (keys %hostlist)){
  $host =~ /^faep(sv|op|cr|pr|kub)\d{1,2}\.$domainname/
      or (warn  "$host is not a supported host" and next);

  if( ! -f "$keyfiledir/$host" ){
    my $princ = "host/$host";
    print STDERR "\n# Generating krb5 hostkeys for $princ\n";
    my $kadmin = `/usr/bin/kadmin -p ank/admin -k -t /etc/krb5.keytab.ank -q "getprinc $princ" 2>&1`;
    "$?" >> 8 and die "Unable to execute kadmin: ", $kadmin, "," , $? >> 8;
    if($kadmin =~ /Principal does not exist/){
      print STDERR "\nKRB5: create new pricipal $princ\n";
      system ("/usr/bin/kadmin",
	      "-p", "ank/admin", "-k",  "-t",  "/etc/krb5.keytab.ank",
	      "-q", "addprinc -pw qwertzuiop $enctype +allow_tix $princ");
      $kadmin = `/usr/bin/kadmin -p ank/admin -k -t /etc/krb5.keytab.ank -q "getprinc $princ" 2>&1 `;
      if($kadmin =~ /Principal does not exist/){
	print STDERR "ERROR: Failed to create principal for $princ\n";
	exit 1;
      }
    }
    print STDERR "\nKRB5: change random password for principal $princ\n";
    system ("/usr/bin/kadmin",
	    "-p", "cpw/admin", "-k",  "-t",  "/etc/krb5.keytab.cpw",
	    "-q", "cpw -randkey $enctype $princ");
    print STDERR "\nKRB5: write keytab file for principal $princ\n";
    system ("/usr/bin/kadmin",
	    "-p", "cpw/admin", "-k", "-t", "/etc/krb5.keytab.cpw",
	    "-q", "ktadd -k $keyfiledir/$host $enctype $princ");
  }
  system("scp",  "$keyfiledir/$host",  "$host:/etc/krb5.keytab");
}
